@@ -79,7 +79,7 @@ class ActionBag(QObject):
              'printAction', 'networkSettingsAction', 'networkHelpAction',
              'selectLeftNodeAction', 'selectRightNodeAction', 'selectUpNodeAction',
              'selectDownNodeAction', 'selectRootNodeAction', 'startSmartConferenceRobot',
-             'stopSmartConferenceRobot')
+             'stopSmartConferenceRobot', 'filterAction')
 
     def __init__(self, parent = None):
         QObject.__init__(self, parent)
@@ -149,6 +149,7 @@ class ActionBag(QObject):
                 QKeySequence.Quit, 'application-exit')
         self.__findAction = self.createAction(self.tr('&Find'), QKeySequence.Find,
                 'edit-find')
+        self.__filterAction = self.createAction(self.tr('F&ilter'))
         self.__settingsAction = self.createAction(self.tr('Settings'),
                 QKeySequence.Preferences, 'preferences-other')
         self.__parentWidthAction = self.createAction(self.tr('Parent'))
@@ -248,7 +249,7 @@ class ActionBag(QObject):
                 self.__editNodeIconsAction, self.__editLabelAction,
                 self.__editEdgeAction, self.__editAffiliationAction,
                 self.__startClientAction, self.__networkSettingsAction,
-                self.__printAction, self.__findAction,
+                self.__printAction, self.__findAction, self.__filterAction,
                 self.__editHyperlinkAction, self.__startServiceAction]
         for action in actionsWithEllipses:
             action.setText(action.text() + '...')
