@@ -88,6 +88,7 @@ class DesktopMainWindow(AbstractMainWindow):
         self._createMainMenu()
         self._createToolBars()
         self.restoreState(QByteArray.fromHex(settings.get('mainWindowState')))
+        self.restoreGeometry(QByteArray.fromHex(settings.get('mainWindowGeometry')))
 
     def setWindowTitle(self, fileName):
         '''
@@ -139,6 +140,7 @@ class DesktopMainWindow(AbstractMainWindow):
         menu.addAction(self._actionBag.redoAction)
         menu.addSeparator()
         menu.addAction(self._actionBag.findAction)
+        menu.addAction(self._actionBag.filterAction)
         menu.addSeparator()
         menu.addAction(self._actionBag.settingsAction)
         menu = self.menuBar().addMenu(self.tr('&Teamwork'))
@@ -190,4 +192,5 @@ class DesktopMainWindow(AbstractMainWindow):
         Close event handler.
         Saves toolbars and dockwidgets state to the settings file'''
         settings.set(mainWindowState = self.saveState().toHex())
+        settings.set(mainWindowGeometry = self.saveGeometry().toHex())
         AbstractMainWindow.closeEvent(self, event)
