@@ -1052,10 +1052,11 @@ class AffiliationDialog(CommonDialog):
     @author: Oleg Kandaurov
     '''
 
-    def __init__(self, model, parent = None):
+    def __init__(self, model, rosterModel, parent = None):
         '''Constructor'''
         CommonDialog.__init__(self, parent)
         self.__model = model
+        self.__rosterModel = rosterModel
         self.setWindowTitle(self.tr('Permissions'))
         self._createComponents()
         self._initComponents()
@@ -1073,12 +1074,16 @@ class AffiliationDialog(CommonDialog):
         horizontalHeader.setStretchLastSection(True)
         self.__jidLabel = QLabel(self.tr('JID'))
         self.__jidEdit = QLineEdit()
+        self.__rosterCombo = QComboBox()
+        self.__rosterCombo.setModel(self.__rosterModel)
+        self.__rosterLabel = QLabel(self.tr('Roster:'))
         self.__affiliationLabel = QLabel(self.tr('Permission'))
         self.__affiliationCombo = QComboBox()
         for data, name in self.__model.mapAffiliations.iteritems():
             self.__affiliationCombo.addItem(name, data)
         self.__additionButton = QPushButton(self.tr('Add participant'))
         self.__additionButton.clicked.connect(self._addParticipant)
+        self.__rosterCombo.activated.connect(self._itemClicked)
 
     def _initComponents(self):
         '''Initialize components'''
@@ -1093,17 +1098,27 @@ class AffiliationDialog(CommonDialog):
         buttonLayout = QVBoxLayout()
         buttonLayout.addWidget(QLabel(''))
         buttonLayout.addWidget(self.__additionButton)
-        additionLayout = QHBoxLayout()
+        rosterLayout = QHBoxLayout()
+        self.__rosterLabel.setAlignment(Qt.AlignCenter)
+        rosterLayout.addWidget(self.__rosterLabel)
+        rosterLayout.addWidget(self.__rosterCombo)
+        additionLayout = QGridLayout()
         additionLayout.setAlignment(Qt.AlignBottom)
-        additionLayout.addLayout(jidLayout)
-        additionLayout.addLayout(affilationLayout)
-        additionLayout.addLayout(buttonLayout)
+        additionLayout.addLayout(jidLayout, 0, 0)
+        additionLayout.addLayout(affilationLayout, 0, 1)
+        additionLayout.addLayout(buttonLayout, 0, 2)
+        additionLayout.addLayout(rosterLayout, 1, 0)
         mainLayout = QVBoxLayout()
         mainLayout.addWidget(self.__view)
         mainLayout.addLayout(additionLayout)
         mainLayout.addWidget(self._buttonBox)
         self.setLayout(mainLayout)
 
+    def _itemClicked(self, index):
+        '''Process clicked signal from rosterCombo'''
+        modelIndex = self.__rosterModel.index(index, 0)
+        self.__jidEdit.setText(modelIndex.data(Qt.EditRole).toString())
+
     def _addParticipant(self):
         '''Retrieve info from components and create participant with given access'''
         jidRegexp = QRegExp(VALID_JID)
