@@ -25,9 +25,9 @@ View delegates for core objects
 '''
 
 import math
-from PyQt4.QtCore import QRectF, QPointF, Qt, QObject, pyqtSignal
+from PyQt4.QtCore import QRectF, QPointF, Qt, QObject, pyqtSignal, QCoreApplication, QString
 from PyQt4.QtGui import QGraphicsScene, QGraphicsItem, QAction, QPen, \
-QPainterPath, QPolygonF, QIcon, QTextDocument, QPainter
+QPainterPath, QPolygonF, QIcon, QTextDocument, QPainter, QGraphicsView
 import hivemind.resource as resource
 import hivemind.settings as settings
 from hivemind.attribute import readable, writable
@@ -119,7 +119,8 @@ class MindMapDelegate(QGraphicsScene):
         #pylint: disable=C0103
         self._addDelegatesForNodeSubtree(node)
         self.positionNodes()
-        self.currentNode = node
+        if not node.parent.folded:
+            self.currentNode = node
         self.__currentLayerDepth = node.depth()
 
     def _addDelegatesForNodeSubtree(self, root):
@@ -130,6 +131,8 @@ class MindMapDelegate(QGraphicsScene):
         '''
         parentDelegate = self.__nodeToDelegate[root.parent]
         newDelegate = NodeDelegate(root, parentDelegate)
+        if root.parent.folded:
+            newDelegate.setVisible(False)
         root.addObserver(self)
         self.__nodeToDelegate[root] = newDelegate
         for child in root.children:
@@ -215,6 +218,8 @@ class MindMapDelegate(QGraphicsScene):
         @param mindMap: mind map to be shown
         @type mindMap: MindMap
         '''
+        if len(self.views()):
+            self.views()[0].setViewportUpdateMode(QGraphicsView.NoViewportUpdate)
         # clear old values
         self.__nodeToDelegate.clear()
         for item in self.items(): self.removeItem(item)
@@ -227,6 +232,8 @@ class MindMapDelegate(QGraphicsScene):
         self.positionNodes()
         self.currentNode = mindMap.root
         self.__currentLayerDepth = 0
+        if len(self.views()):
+            self.views()[0].setViewportUpdateMode(QGraphicsView.SmartViewportUpdate)
         self.mapChanged.emit()
 
     _MAP_MARGIN_WIDTH = 300
@@ -247,6 +254,7 @@ class MindMapDelegate(QGraphicsScene):
         
         @type parentNodeDelegate: NodeDelegate
         '''
+        QCoreApplication.instance().processEvents()
         parentNodeDelegate.node.addObserver(self)
         childNodes = parentNodeDelegate.node.children
         for node in childNodes:
@@ -271,6 +279,7 @@ class MindMapDelegate(QGraphicsScene):
         '''Update node and it's subtree bounding rectangle'''
         self.__nodeToDelegate[node].updateNodeBounds()
         self.positionNodes()
+        self.__nodeToDelegate[node].updateSubtree()
 
     def getDelegate(self, node):
         '''
@@ -287,7 +296,7 @@ class MindMapDelegate(QGraphicsScene):
         @rtype: NodeDelegate
         '''
         for item in self.items(position):
-            if item.textRectOnScene().contains(position):
+            if isinstance(item, NodeDelegate) and item.textRectOnScene().contains(position):
                 return item
         return None
 
@@ -332,10 +341,10 @@ class MindMapDelegate(QGraphicsScene):
                     position, self.__rootDelegate): continue
             child = self.getDelegate(node)
             if prev is None:
-                if position.y() < child.textRectOnScene().center().y():
+                if position.y() <= child.textRectOnScene().center().y():
                     return None, child
-            elif position.y() > prev.textRectOnScene().center().y() and \
-                    position.y() < child.textRectOnScene().center().y():
+            elif position.y() >= prev.textRectOnScene().center().y() and \
+                    position.y() <= child.textRectOnScene().center().y():
                 return prev, child
             prev = child
         return prev, None
@@ -385,7 +394,7 @@ class MindMapDelegate(QGraphicsScene):
         if children is None: return None
         parents = set()
         for child in children:
-            if (child.parentItem()) is not None:
+            if (child.parentItem()) is not None and isinstance(child, NodeDelegate):
                 parents.add(child.parentItem())
             else:
                 parents.add(self.__rootDelegate)
@@ -421,20 +430,17 @@ class MindMapDelegate(QGraphicsScene):
         @param leftSubtree: select left node if true.
         '''
         nextNode = None
-        if self.currentNode.parent is None:
-            #node is root
-            if not self.currentNode.folded:
-                #node is not folded
-                #find first left(right) child
-                for node in self.currentNode.children:
-                    if node.left == leftSubtree:
-                        nextNode = node
-                        break
-        elif self.currentNode.left == leftSubtree:
-            #node is not root. Select first child
+        if self.currentNode.left == leftSubtree or self.currentNode.parent is None:
             if self.currentNode.children and not self.currentNode.folded:
                 #node has children and is not folded
-                nextNode = self.currentNode.children[0]
+                currentDelegate = self.getDelegate(self.currentNode)
+                position = currentDelegate.textRectOnScene().center()
+                if self.currentNode.parent is None:
+                    position.setX(position.x() + (-1 if leftSubtree else 1))
+                above, below = self._neighbourDelegates(position, currentDelegate)
+                nearest = MindMapDelegate.nearestDelegateByOrdinate(position, above,
+                        below)
+                nextNode = nearest.node
         else:
             #select parent
             nextNode = self.currentNode.parent
@@ -527,6 +533,7 @@ class NodeDelegate(QGraphicsItem):
         @type highlighted: boolean
         '''
         self.__highlighted = highlighted
+        self.update()
 
     highlighted = property(_isHighlighted, _setHighlighted, None, 'Highlighted flag')
 
@@ -544,11 +551,11 @@ class NodeDelegate(QGraphicsItem):
         self.__highlighted = False
         self.__selected = False
         self.__positionChanged = True
-        self.__linkPoint = None
-        self.__parentLinkPoint = None
         self.__textBoundingRect = None
         self.__boundingRect = None
         QGraphicsItem.__init__(self, parent)
+        self.__edge = Edge(self.__node, self)
+        self.setCacheMode(QGraphicsItem.DeviceCoordinateCache)
         self.updateNodeBounds(False)
         self._updateNodePosition()
 
@@ -600,11 +607,23 @@ class NodeDelegate(QGraphicsItem):
         @return: list of child delegates that belong to specified tree
         @rtype: list
         '''
-        subtree = [item for item in self.childItems()
-                       if item.node.left is left]
+        subtree = [item for item in self._childDelegates() if item.node.left is left]
         subtree.sort(key = lambda delegate: self.__node.children.index(delegate.node))
         return subtree
 
+    def _childDelegates(self):
+        '''Get all child node delegates of the current delegate, filter out other elements'''
+        return [item for item in self.childItems() if isinstance(item, NodeDelegate)]
+
+    def updateSubtree(self):
+        '''
+        Update view of the delegate and it subtree
+        '''
+        self.update()
+        for child in self._childDelegates():
+            child.updateSubtree()
+
+
     def _computeSoloVerticalSpace(self):
         '''
         Compute minimum vertical space needed for the current node
@@ -641,7 +660,7 @@ class NodeDelegate(QGraphicsItem):
         @rtype: float
         '''
         if child.node.left:
-            return - NodeDelegate.__NODE_HORIZONTAL_SPACING \
+            return -NodeDelegate.__NODE_HORIZONTAL_SPACING \
                 - child.textBoundingRect().width() - child.linkPointOffset()
         else:
             return NodeDelegate.__NODE_HORIZONTAL_SPACING \
@@ -677,11 +696,12 @@ class NodeDelegate(QGraphicsItem):
         if self.x() != x or self.y() != y:
             QGraphicsItem.setPos(self, x, y)
             self._updateNodePosition()
+            self.update()
 
     def _updateNodePosition(self):
         '''Calculate new node position-aware properties'''
         if self.parentItem() is not None:
-            self._calculateParentLinkPoint()
+            self.__edge.calculateParentLinkPoint()
         self._calculateBoundingRect()
 
     def updateNodeBounds(self, propagate = True):
@@ -691,35 +711,12 @@ class NodeDelegate(QGraphicsItem):
         @type propagate: bool
         '''
         self._calculateTextBoundingRect()
-        self._calculateLinkPoint()
+        self.__edge.calculateLinkPoint()
         self._updateNodePosition()
         if propagate:
-            for delegate in self.childItems():
+            for delegate in self._childDelegates():
                 delegate.updateNodeBounds(self)
 
-    def _calculateLinkPoint(self):
-        '''Calculate connection point for joint link'''
-        if self.__node.parent is None:
-            self.__linkPoint = None
-            return
-        linkPoint = QPointF()
-        if self.__node.effectiveStyle == 'bubble':
-            linkPoint.setY(self.textBoundingRect().height() / 2.0)
-        else:
-            linkPoint.setY(self.textBoundingRect().height() -
-                           self.__node.effectiveEdgeWidth / 2.0)
-        if self.__node.left:
-            linkPoint.setX(self.textBoundingRect().width())
-        self.__linkPoint = linkPoint
-
-    @property
-    def linkPoint(self):
-        '''
-        Return connection point for joint link
-        @rtype: QPointF
-        '''
-        return None if self.__linkPoint is None else QPointF(self.__linkPoint)
-
     def paint(self, painter, option, widget):
         '''
         Paint node content and its joint link
@@ -762,8 +759,6 @@ class NodeDelegate(QGraphicsItem):
         painter.setBrush(Qt.NoBrush)
         # paint icons and text content
         self._paintNodeContent(painter)
-        # paint join link from parent to self
-        self._paintEdge(painter, edgeStyle)
         # paint folded mark (circle)
         if self.__node.folded:
             self._paintFoldedMark(painter, boundRect)
@@ -772,12 +767,13 @@ class NodeDelegate(QGraphicsItem):
         ''' Paint text label of node'''
         painter.setFont(self.__node.effectiveLabelFont)
         painter.setPen(self.__node.effectiveLabelColor)
-        labelPoint = self.linkPoint
+        labelPoint = QPointF(self.__edge.linkPoint)
         if self.__node.left:
-            labelPoint.setX(labelPoint.x() + NodeDelegate.__LABEL_MARGIN)
+            labelPoint.setX(labelPoint.x() - self.linkPointOffset() \
+                     + NodeDelegate.__LABEL_MARGIN)
         else:
-            labelPoint.setX(labelPoint.x() - self.__node.labelSize.width() - \
-                    NodeDelegate.__LABEL_MARGIN)
+            labelPoint.setX(labelPoint.x() - self.linkPointOffset() \
+                     - self.__node.labelSize.width() - NodeDelegate.__LABEL_MARGIN)
         labelPoint.setY(labelPoint.y() - self.__node.effectiveEdgeWidth / 2.0 - 2.0)
         painter.drawText(labelPoint, self.__node.labelText)
         pen = QPen(self.__node.effectiveEdgeColor, self.__node.effectiveEdgeWidth,
@@ -785,9 +781,9 @@ class NodeDelegate(QGraphicsItem):
         if self.__node.effectiveEdgeStyle.startswith('sharp') and not self.__node.children:
             pen.setWidthF(0.5)
         painter.setPen(pen)
-        linkPoint = self.linkPoint
-        painter.drawLine(linkPoint, QPointF(linkPoint.x() + self.linkPointOffset(),
-                linkPoint.y()))
+        linkPoint = QPointF(self.__edge.linkPoint)
+        painter.drawLine(QPointF(linkPoint.x() - self.linkPointOffset(), linkPoint.y())
+                , linkPoint)
 
     def _paintRoot(self, painter, boundRect):
         '''
@@ -809,7 +805,10 @@ class NodeDelegate(QGraphicsItem):
         '''
         painter.setPen(QPen(self.__node.effectiveEdgeColor, 1.0))
         painter.setBrush(Qt.white)
-        markPoint = self.linkPoint
+        if self.__edge.linkPoint is None:
+            markPoint = None
+        else:
+            markPoint = QPointF(self.__edge.linkPoint)
         if markPoint is None: # root
             markPoint = QPointF()
             markPoint.setY(boundRect.height() / 2.0)
@@ -819,7 +818,8 @@ class NodeDelegate(QGraphicsItem):
             if any(node.left for node in self.__node.children):
                 markPoint.setX(0.0)
         else: # child node
-            markPoint.setX(0.0 if markPoint.x() else boundRect.width())
+            markPoint.setX(0.0 if markPoint.x() - self.linkPointOffset() \
+                     else boundRect.width())
         painter.drawEllipse(markPoint, 4.0, 4.0)
 
     def _rootBoundingRect(self, size):
@@ -924,20 +924,20 @@ class NodeDelegate(QGraphicsItem):
         '''Calculate bounding rectangle'''
         if self.parentItem() is None:
             self.__boundingRect = self.textBoundingRect()
+            if self.__node.folded:
+                self.__boundingRect.setX(self.__boundingRect.x() \
+                         - NodeDelegate.__FOLD_MARK_SIZE)
+                self.__boundingRect.setWidth(self.__boundingRect.width() \
+                         + NodeDelegate.__FOLD_MARK_SIZE * 2)
             return
-        linkPoint = QPointF(self.__parentLinkPoint)
         textBounds = self.textBoundingRect()
-        left = min(linkPoint.x(), textBounds.x())
-        top = min(linkPoint.y(), textBounds.y(), -self._labelOverhead())
+        left = textBounds.x()
         if not self.__node.left:
-            width = textBounds.width() + math.fabs(linkPoint.x()) + textBounds.x()
-        else:
-            width = linkPoint.x() - textBounds.x()
-            if self.__node.folded: left -= NodeDelegate.__FOLD_MARK_SIZE
-        if linkPoint.y() < textBounds.y():
-            height = textBounds.height() + math.fabs(top) + textBounds.y()
-        else:
-            height = max(math.fabs(top) + textBounds.height(), linkPoint.y() - textBounds.y())
+            left += self.linkPointOffset()
+        top = min(textBounds.y(), -self._labelOverhead())
+        width = textBounds.width() + abs(self.linkPointOffset())
+        if self.__node.folded and self.__node.left: left -= NodeDelegate.__FOLD_MARK_SIZE
+        height = max(textBounds.height(), self.__node.labelSize.height())
         if self.__node.folded:
             width += NodeDelegate.__FOLD_MARK_SIZE
             if self.__node.effectiveStyle == 'fork': height += NodeDelegate.__FOLD_MARK_SIZE
@@ -995,29 +995,84 @@ class NodeDelegate(QGraphicsItem):
                 self.__node.effectiveEdgeWidth / 2.0 - 2.0
         if self.__node.effectiveStyle == 'bubble':
             labelTop -= textBounds.height() / 2.0
-        return - labelTop if labelTop < 0.0 else 0.0
+        return -labelTop if labelTop < 0.0 else 0.0
 
-    def _calculateParentLinkPoint(self):
+    def linkPointOffset(self):
+        '''
+        @return: offset of edge link point in pixels specified
+                 by label width and its margins
+        @rtype: float
+        '''
+        labelWidth = self.__node.labelSize.width()
+        if labelWidth == 0.0:
+            return 0.0
+        offset = labelWidth + NodeDelegate.__LABEL_MARGIN * 2.0
+        return offset if self.__node.left else -offset
+
+
+class Edge(QGraphicsItem):
+    '''
+    Edge delegate
+
+    @authors: Andrew Golovchenko, Andrew Vasilev, Alexander Kulikov, Eldar Mamedov
+    '''
+
+    readable('parentLinkPoint', 'linkPoint')
+
+    def __init__(self, node, parent = None):
+        '''
+        Initialize delegate with specified node
+
+        @param node: delegated Node object
+        @type node: Node
+        @param parent: parent object of this delegate
+        @type parent: NodeDelegate
+        '''
+        self.__node = node
+        self.__parentLinkPoint = None
+        self.__linkPoint = None
+        QGraphicsItem.__init__(self, parent)
+
+    def calculateParentLinkPoint(self):
         '''Calculate connection point to parent node'''
         parentLinkPoint = QPointF()
-        parentRect = self.parentItem().textBoundingRect()
+        nodeDelegate = self.parentItem()
+        parentRect = nodeDelegate.parentItem().textBoundingRect()
         if not self.__node.left: parentLinkPoint.setX(parentRect.width())
-        parentNode = self.parentItem().node
+        parentNode = nodeDelegate.parentItem().node
         # root node never has fork style and only root has no parent
         if parentNode.effectiveStyle == 'bubble' or parentNode.parent is None:
             parentLinkPoint.setY(parentRect.height() / 2.0)
         else:
             parentRect.setHeight(parentRect.height() - parentNode.effectiveEdgeWidth / 2.0)
             parentLinkPoint.setY(parentRect.height())
-        self.__parentLinkPoint = self.mapFromParent(parentLinkPoint)
+        self.__parentLinkPoint = nodeDelegate.mapFromParent(parentLinkPoint)
 
-    def _paintEdge(self, painter, style):
+    def calculateLinkPoint(self):
+        '''Calculate connection point for joint link'''
+        if self.__node.parent is None:
+            self.__linkPoint = None
+            return
+        linkPoint = QPointF()
+        nodeDelegate = self.parentItem()
+        if self.__node.effectiveStyle == 'bubble':
+            linkPoint.setY(nodeDelegate.textBoundingRect().height() / 2.0)
+        else:
+            linkPoint.setY(nodeDelegate.textBoundingRect().height() -
+                           self.__node.effectiveEdgeWidth / 2.0)
+        linkPoint.setX(nodeDelegate.linkPointOffset())
+        if self.__node.left:
+            linkPoint.setX(linkPoint.x() + nodeDelegate.textBoundingRect().width())
+        self.__linkPoint = linkPoint
+
+    def paint(self, painter, option, widget):
         '''
         Paint join link from parent node to current
         @type painter: QPanter
         @param style: style of node join link
         @type style: string
         '''
+        if self.__linkPoint is None: return
         paintEdge = {'bezier': self._paintBezierEdge,
                 'sharp_bezier': self._paintSharpBezierEdge,
                 'linear': self._paintLinearEdge,
@@ -1025,9 +1080,8 @@ class NodeDelegate(QGraphicsItem):
         painter.setPen(QPen(self.__node.effectiveEdgeColor, self.__node.effectiveEdgeWidth,
                 Qt.SolidLine, Qt.FlatCap, Qt.RoundJoin))
         startPoint = QPointF(self.__parentLinkPoint)
-        endPoint = self.linkPoint
-        endPoint.setX(endPoint.x() + self.linkPointOffset())
-        paintEdge[style](painter, startPoint, endPoint)
+        endPoint = QPointF(self.__linkPoint)
+        paintEdge[self.__node.effectiveEdgeStyle](painter, startPoint, endPoint)
 
     def _paintBezierEdge(self, painter, startPoint, endPoint):
         '''
@@ -1092,17 +1146,15 @@ class NodeDelegate(QGraphicsItem):
         painter.drawPath(curvePath)
         painter.fillPath(curvePath, self.__node.effectiveEdgeColor)
 
-    def linkPointOffset(self):
-        '''
-        @return: offset of edge link point in pixels specified
-                 by label width and its margins
-        @rtype: float
-        '''
-        labelWidth = self.__node.labelSize.width()
-        if labelWidth == 0.0:
-            return 0.0
-        offset = labelWidth + NodeDelegate.__LABEL_MARGIN * 2.0
-        return offset if self.__node.left else - offset
+    def boundingRect(self):
+        '''Bounding rectangle of the edge'''
+        if self.__linkPoint is None: return QRectF()
+        x = min(self.__parentLinkPoint.x(), self.__linkPoint.x())
+        y = min(self.__parentLinkPoint.y(), self.__linkPoint.y())
+        width = abs(self.__parentLinkPoint.x() - self.__linkPoint.x())
+        height = abs(self.__parentLinkPoint.y() - self.__linkPoint.y())
+        return QRectF(x, y, width, height)
+
 
 class NodeLocation(object):
     '''
@@ -1137,6 +1189,155 @@ class NodeLocation(object):
         return self.__position is not None or self.__right is not None
 
 
+class Finder(QObject):
+    '''
+    Find nodes that match a criteria
+    @author: Oleg Kandaurov
+    '''
+
+    def __init__(self):
+        '''Constructor'''
+        QObject.__init__(self)
+        self.__processCondition = None
+        self.__nodeMap = {}
+
+    def findNodes(self, node, processCondition):
+        '''
+        Find all children of given node that match the condition
+        @param node: Find from the Node
+        @param processCondition: Function that provide a condition
+        @return: list of Node
+        '''
+        self.__processCondition = processCondition
+        self.__nodeMap = {}
+        self._find(node, 0)
+
+    def createSortedList(self):
+        '''
+        Create list of found nodes sorted from left to right
+        @return: list of found nodes
+        '''
+        foundNodes = []
+        keys = self.__nodeMap.keys()
+        keys.sort()
+        for key in keys:
+            foundNodes.extend(self.__nodeMap[key])
+        return foundNodes
+
+    def includeParentNodes(self):
+        '''Include parents nodes of found nodes'''
+        depths = self.__nodeMap.keys()
+        depths.remove(0)
+        depths.sort(lambda x, y: cmp (abs(y), abs(x)))
+        for depth in depths:
+            for node in self.__nodeMap[depth]:
+                upperLevel = depth + 1 if node.left else depth - 1
+                if node.parent not in self.__nodeMap[upperLevel]:
+                    self.__nodeMap[upperLevel].append(node.parent)
+
+    def _find(self, node, depth):
+        '''
+        Recursive function to find nodes
+        @type node: Node
+        @type depth: int
+        '''
+        if depth not in self.__nodeMap:
+            self.__nodeMap[depth] = []
+        if self.__processCondition(node):
+            self.__nodeMap[depth].append(node)
+        for childrenNode in node.children:
+            deeperLevel = depth - 1 if childrenNode.left else depth + 1
+            self._find(childrenNode, deeperLevel)
+
+
+class MindMapFilter(QObject):
+    '''
+    Filters mind map
+    @author: Oleg Kandaurov
+    '''
+
+    def __init__(self, controller):
+        '''Constructor'''
+        QObject.__init__(self)
+        self.__controller = controller
+        self.__mindMapScene = controller.mindMapScene
+        self.__finder = Finder()
+        self.__foundNodes = []
+        self.__filterMode = False
+        self.__targetText = QString()
+        self.__mindMapScene.curMapChanged.connect(self.refresh)
+        self.__mindMapScene.mapChanged.connect(self.refresh)
+
+    def setTargetText(self, targetText):
+        '''Setter'''
+        self.__targetText = targetText
+
+    def _condition(self, node):
+        '''
+        Decide whether node match a condition or not
+        @param node: Node to be inspected
+        @return: True if node matches a condition
+        '''
+        if not len(self.__targetText):
+            return False
+        nodeText = QTextDocument()
+        nodeText.setHtml(node.text)
+        if self.__targetText.toLower() in nodeText.toPlainText().toLower():
+            return True
+        return False
+
+    def _filter(self, node, isVisible):
+        '''
+        Filter given node
+        @param node: Node that need to be examined
+        @param isVisible: False if the node ancestor is folded
+        '''
+        nodeDelegate = self.__mindMapScene.getDelegate(node)
+        if nodeDelegate is None: return
+        if node in self.__foundNodes and self.__filterMode:
+            self.__controller.showNode(node)
+            nodeDelegate.setVisible(True)
+        elif isVisible:
+            nodeDelegate.setVisible(not self.__filterMode)
+
+    def _filterNodes(self):
+        '''Apply filter to all nodes'''
+        def traverse(node, isVisible):
+            '''
+            Traverse through children nodes
+            @param node: Parent node
+            @param isVisible: False if the node ancestor is folded
+            '''
+            for node in node.children:
+                if isVisible: isVisible = node.folded is None
+                self._filter(node, isVisible)
+                traverse(node, isVisible)
+
+        traverse(self.__mindMapScene.rootDelegate.node, True)
+
+    def toggleFilter(self, filterMode):
+        '''
+        Update filter trigger
+        @param filterMode: Indicates that filtering is enabled
+        '''
+        self.__filterMode = filterMode
+        if not filterMode:
+            self._filterNodes()
+        else:
+            self._updateFilteredNodes()
+
+    def _updateFilteredNodes(self):
+        '''Find nodes and filter it'''
+        self.__finder.findNodes(self.__mindMapScene.rootDelegate.node, self._condition)
+        self.__finder.includeParentNodes()
+        self.__foundNodes = self.__finder.createSortedList()
+        self._filterNodes()
+
+    def refresh(self):
+        '''Refresh all filtered nodes'''
+        self._updateFilteredNodes()
+
+
 class NodeFinder(QObject):
     '''
     Node finder
@@ -1150,14 +1351,32 @@ class NodeFinder(QObject):
         @type controller: MainWindowController
         '''
         QObject.__init__(self)
+        self.__finder = Finder()
         self.__foundNodes = []
         self.__index = -1
         self.__controller = controller
+        self.__mindMapScene = controller.mindMapScene
         self.__findText = None
         self.__caseSensitive = False
         self.__highlight = False
-        controller.mindMapScene.curMapChanged.connect(self.refresh)
-        controller.mindMapScene.mapChanged.connect(self.refresh)
+        self.__mindMapScene.curMapChanged.connect(self.refresh)
+        self.__mindMapScene.mapChanged.connect(self.refresh)
+
+    def _condition(self, node):
+        '''
+        Decide whether node match a condition or not
+        @param node: Node to be inspected
+        @return: True if node matches a condition
+        '''
+        text = QTextDocument()
+        text.setHtml(node.text)
+        if self.__caseSensitive:
+            if self.__findText in text.toPlainText():
+                return True
+        else:
+            if self.__findText.toLower() in text.toPlainText().toLower():
+                return True
+        return False
 
     def prevFoundNode(self):
         '''
@@ -1169,7 +1388,7 @@ class NodeFinder(QObject):
                 self.__index = len(self.__foundNodes) - 1
             foundNode = self.__foundNodes[self.__index]
             self.__controller.showNode(foundNode)
-            self.__controller.mindMapScene.currentNode = foundNode
+            self.__mindMapScene.currentNode = foundNode
         self._status()
 
     def nextFoundNode(self):
@@ -1182,7 +1401,7 @@ class NodeFinder(QObject):
                 self.__index = 0
             foundNode = self.__foundNodes[self.__index]
             self.__controller.showNode(foundNode)
-            self.__controller.mindMapScene.currentNode = foundNode
+            self.__mindMapScene.currentNode = foundNode
         self._status()
 
     def highlightNodes(self, highlight = False):
@@ -1192,13 +1411,12 @@ class NodeFinder(QObject):
         '''
         self.__highlight = highlight
         for node in self.__foundNodes:
-            nodeDelegate = self.__controller.mindMapScene.getDelegate(node)
+            nodeDelegate = self.__mindMapScene.getDelegate(node)
             if nodeDelegate is None: continue
             if highlight:
                 nodeDelegate.highlighted = True
             else:
                 nodeDelegate.highlighted = False
-        self.__controller.mindMapScene.update()
 
     def findNodes(self, findText = None, caseSensitive = False, highlight = False):
         '''
@@ -1207,39 +1425,15 @@ class NodeFinder(QObject):
         @type caseSensitive: bool
         @type highlight: bool
         '''
-        def find(node, depth):
-            '''
-            Recursive function to find nodes
-            @type node: Node
-            @type depth: int
-            '''
-            if depth not in nodeMap:
-                nodeMap[depth] = []
-            text.setHtml(node.text)
-            if caseSensitive:
-                if findText in text.toPlainText():
-                    nodeMap[depth].append(node)
-            else:
-                if findText.toLower() in text.toPlainText().toLower():
-                    nodeMap[depth].append(node)
-            for childrenNode in node.children:
-                if childrenNode.left:
-                    find(childrenNode, depth - 1)
-                else:
-                    find(childrenNode, depth + 1)
+
         if self.__highlight: self.highlightNodes(False)
         self.__foundNodes = []
         self.__index = -1
-        nodeMap = {}
-        text = QTextDocument()
         self.__findText = findText
         self.__caseSensitive = caseSensitive
-        if not len(findText): return
-        find(self.__controller.mindMapScene.rootDelegate.node, 0)
-        keys = nodeMap.keys()
-        keys.sort()
-        for key in keys:
-            self.__foundNodes.extend(nodeMap[key])
+        if findText is None or not len(findText): return
+        self.__finder.findNodes(self.__mindMapScene.rootDelegate.node, self._condition)
+        self.__foundNodes = self.__finder.createSortedList()
         if highlight: self.highlightNodes(highlight)
         self._status()
 
