@@ -39,7 +39,7 @@ import re
 from twisted.python import log
 from PyQt4.QtGui import QActionGroup, QGraphicsBlurEffect, QMessageBox, QUndoStack, \
 QFileDialog, QPixmap, QPainter, QAction, QDesktopServices, QPrintDialog, QPrinter, \
-QUndoCommand
+QUndoCommand, QProgressDialog
 from PyQt4.QtCore import QObject, SIGNAL, QTimer, QLocale, QTranslator, QLibraryInfo, \
 Qt, QRectF, QUrl
 import hivemind.gui_factory as gui_factory
@@ -103,8 +103,24 @@ class MainWindowController(QObject):
                 adaptCallable(self.onItemClicked))
         self.__mainWindow.closeEvnt.connect(self._closeEvent)
         splashScreen.finish(self.__mainWindow)
-        self.__mainWindow.showNormal()
+        self.__mainWindow.show()
         _configureApplication(self.__mindMapController)
+        self.__findWidget = gui_factory.createFindToolBar(self)
+        self.__filterWidget = gui_factory.createFilterToolBar(self)
+        self._addToolbar(self.__filterWidget)
+        self._addToolbar(self.__findWidget)
+
+    def _addToolbar(self, toolbar):
+        '''
+        Add toolbar according to current platform
+        @type toolbar: QWidget
+        '''
+        if sysconfig.PLATFORM == 'maemo':
+            toolbar.move(self.__mainWindow.width() - toolbar.width() + 10,
+                    self.__mainWindow.height() - toolbar.height() + 10)
+        else:
+            self.__mainWindow.addToolBar(Qt.BottomToolBarArea, toolbar)
+        toolbar.setVisible(False)
 
     def _setActionsState(self, currentNode):
         '''
@@ -221,6 +237,7 @@ class MainWindowController(QObject):
         self.__actionBag.exitAction.triggered.connect(self._closeMainWindow)
         self.__actionBag.settingsAction.triggered.connect(self.onSettings)
         self.__actionBag.findAction.triggered.connect(self.onFind)
+        self.__actionBag.filterAction.triggered.connect(self.onFilter)
         self.__actionBag.showContextMenuAction.triggered.connect(self._virtualClick)
         self.__actionBag.editHyperlinkAction.triggered.connect(self.editHyperlink)
         self.__actionBag.openHyperlinkAction.triggered.connect(self.openHyperlink)
@@ -425,21 +442,16 @@ class MainWindowController(QObject):
         gui_factory.createSettingsDialog().exec_()
 
     def onFind(self):
-        '''
-        Process find action
-        '''
-        if self.__findWidget is None:
-            self.__findWidget = gui_factory.createFindToolBar(self)
-            if sysconfig.PLATFORM == 'maemo':
-                self.__findWidget.move(self.__mainWindow.width() - self.__findWidget.width() +
-                        10, self.__mainWindow.height() - self.__findWidget.height() + 10)
-            else:
-                self.__mainWindow.addToolBar(Qt.BottomToolBarArea, self.__findWidget)
-        if self.__findWidget.isVisible():
-            self.__findWidget.setVisible(False)
-        else:
-            self.__findWidget.setVisible(True)
-            self.__findWidget.setDefaultFocus()
+        '''Process find action'''
+        self.__filterWidget.setVisible(False)
+        self.__findWidget.setVisible(not self.__findWidget.isVisible())
+        self.__findWidget.setDefaultFocus()
+
+    def onFilter(self):
+        '''Process filter action'''
+        self.__findWidget.setVisible(False)
+        self.__filterWidget.setVisible(not self.__filterWidget.isVisible())
+        self.__filterWidget.setDefaultFocus()
 
     def editEdge(self):
         '''Edit node's edge decorations and style'''
@@ -757,7 +769,7 @@ class MindMapController(QObject):
         '''
         Deserialize data according to the type
         @param data: serialized data
-        @param type: type of data
+        @param dataType: type of data
         @return: deserialized data
         '''
         if dataType == 'command':
@@ -878,15 +890,21 @@ class MindMapController(QObject):
             filePath = self._generateAutosaveFilename(path)
         else:
             self.__autosaved = False
+        progressDialog = QProgressDialog(gui_factory.defaultParent())
+        progressDialog.setWindowTitle(self.tr('Please wait...'))
+        progressDialog.setMaximum(0)
+        progressDialog.show()
         try:
             mindMap = parser.loadMindMap(filePath) if filePath is not None else MindMap()
         except Exception:
             QMessageBox.critical(gui_factory.defaultParent(), self.tr('File open error'),
                     unicode(self.tr('Cannot read mind map from file %s')) %
                     os.path.split(path)[1])
+            progressDialog.close()
             return
         self._setFileName(path)
         self.loadMindMap(mindMap, False)
+        progressDialog.close()
         settings.addRecentFile(path)
 
     def loadMindMap(self, mindMap, resetName = True):
