@@ -32,12 +32,12 @@ QAction, QUndoCommand, QLineEdit, QSpinBox, QCheckBox, QComboBox, QLabel, \
 QToolButton, QIcon, QApplication, QFormLayout, QFontComboBox, QFontDatabase, \
 QHBoxLayout, QGraphicsBlurEffect, QFrame, QPalette, QPushButton, \
 QColorDialog, QPainter, QKeyEvent, QPixmap, QListView, QKeySequence, QToolBar, \
-QFont, QButtonGroup, QRadioButton
+QFont, QButtonGroup, QRadioButton, QColor, QPen, QBrush
 from hivemind.attribute import readable
 from PyQt4.QtCore import QAbstractAnimation, QObject, SIGNAL, QLocale, Qt, \
 QRect, QEvent, QRectF, QSize, QParallelAnimationGroup, QPropertyAnimation, \
 QPointF, QRegExp, QSequentialAnimationGroup, QEasingCurve, QTimer, pyqtSignal
-from hivemind.gui_delegates import NodeFinder
+from hivemind.gui_delegates import NodeFinder, MindMapFilter
 from hivemind.network.models import AffiliationModel
 
 
@@ -1831,18 +1831,125 @@ class MaemoSidePanel(QObject):
         self.__hideAnimation.finished.connect(self._toggleSidebarStatus)
 
 
-class FindToolBar(SearchingTrMixin, QToolBar):
+class MaemoToolBar(QToolBar):
     '''
-    Abstract class for find toolBar
+    Toolbar for Maemo platform
+    @author: Eldar Mamedov, Oleg Kandaurov
     '''
-    def __init__(self, controller, parent = None):
+
+    __HEIGHT = 82
+    '''Height of the tool bar'''
+    __WIDTH = 630
+    '''Width of the tool bar'''
+    __RADIUS = 20
+    '''Corner radius'''
+
+    def __init__(self, parent = None):
         '''
-        @type controller: MainWindowController
         @type parent: QWidget
         '''
         QToolBar.__init__(self, parent)
+        self.resize(self.__WIDTH, self.__HEIGHT)
+        self.setAllowedAreas(Qt.NoToolBarArea)
+        self.setMovable(False)
+        self.setFloatable(False)
+        parent.installEventFilter(self)
+
+    def paintEvent(self, paintEvent):
+        '''
+        Paint black background
+        @param paintEvent: paint event representation
+        @type paintEvent: QPaintEvent
+        '''
+        painter = QPainter(self)
+        background = QColor(self.backgroundRole())
+        background.setAlpha(128)
+        brush = QBrush(background)
+        pen = QPen(background)
+        painter.fillRect(0, self.__RADIUS, self.__WIDTH, self.__HEIGHT - self.__RADIUS, brush)
+        painter.fillRect(self.__RADIUS, 0, self.__WIDTH - self.__RADIUS, self.__RADIUS, brush)
+        painter.setPen(pen)
+        painter.setBrush(brush)
+        painter.drawPie(0, 0, (self.__RADIUS - 1) * 2, (self.__RADIUS - 1) * 2, 180 * 8,
+                180 * 8)
+
+    def eventFilter(self, obj, event):
+        '''
+        Catch Windows state changed and Resize events from parent object
+        @type obj: QEvent
+        @type event: QEvent
+        '''
+        parent = self.parentWidget()
+        if obj != self.parent():
+            return QToolBar.eventFilter(self, obj, event)
+        isFullScreen = parent.windowState() & Qt.WindowFullScreen
+        if event.type() == QEvent.Resize:
+            if isFullScreen:
+                self.resize(self.width() - parent.fullScreenExitButton.width(), self.__HEIGHT)
+                self.move(parent.width() - self.width() - parent.fullScreenExitButton.width(),
+                         parent.height() - parent.fullScreenExitButton.height())
+            else:
+                self.resize(self.__WIDTH, self.__HEIGHT)
+                self.move(parent.width() - self.width() + 10, parent.height() - self.height()
+                        + 10)
+        return QToolBar.eventFilter(self, obj, event)
+
+
+class FilterToolBar(SearchingTrMixin, QObject):
+    '''
+    Abstract class for filter toolbar
+    @author: Oleg Kandaurov
+    '''
+    def __init__(self, controller):
+        '''
+        @type controller: MainWindowController
+        '''
+        QObject.__init__(self)
+        SearchingTrMixin.__init__(self)
+        self._finder = MindMapFilter(controller)
+        self._closeButton = QPushButton()
+
+    def setDefaultFocus(self):
+        '''Set default focus'''
+        self._findLineEdit.setFocus()
+
+    def _createComponents(self):
+        '''Create components'''
+        #pylint: disable = W0201
+        self._findWidget = QWidget()
+        self._findLineEdit = QLineEdit()
+        self._findLineEdit.textEdited.connect(self._updateParams)
+        self._filterButton = QPushButton(self.tr('Filter'))
+        self._filterButton.setCheckable(True)
+        self._filterButton.toggled.connect(self._finder.toggleFilter)
+        self._closeAction = QAction(self)
+
+    def _createLayout(self):
+        '''Create layout'''
+        mainLayout = QHBoxLayout()
+        mainLayout.addWidget(self._findLineEdit)
+        mainLayout.addWidget(self._filterButton)
+        mainLayout.addStretch()
+        mainLayout.addWidget(self._closeButton)
+        self._findWidget.setLayout(mainLayout)
+
+    def _updateParams(self):
+        '''Update filtered nodes'''
+        self._finder.setTargetText(self._findLineEdit.text())
+        self._finder.refresh()
+
+
+class FindToolBar(SearchingTrMixin, QObject):
+    '''
+    Abstract class for find toolbars
+    @author: Eldar Mamedov, Oleg Kandaurov
+    '''
+    def __init__(self, controller):
+        '''
+        @type controller: MainWindowController
+        '''
+        QObject.__init__(self)
         SearchingTrMixin.__init__(self)
-        self.setWindowTitle(self.tr('Find'))
         self._finder = NodeFinder(controller)
 
     def setDefaultFocus(self):
@@ -1888,4 +1995,3 @@ class FindToolBar(SearchingTrMixin, QToolBar):
         if status == 'not_found': self._statusLabel.setText(self.tr('No found nodes'))
         if status == 'first': self._statusLabel.setText(self.tr('First found node'))
         if status == 'last': self._statusLabel.setText(self.tr('Last found node'))
-
