@@ -26,14 +26,14 @@ Dialogs for Maemo platform
 
 import hivemind.resource as resource
 from PyQt4.QtGui import QFrame, QWidget, QGridLayout, QStackedWidget, QVBoxLayout, \
-QRadioButton, QHBoxLayout, QToolBar, QIcon, QPalette, QPushButton, QLabel, \
-QTextBrowser, QDialog, QFont, QDesktopServices, \
-QAction, QPainter, QBrush, QPen, QColor
-from PyQt4.QtCore import Qt, QSize, QTimer, QUrl, QEvent
+    QRadioButton, QHBoxLayout, QToolBar, QIcon, QPalette, QPushButton, QLabel, \
+    QTextBrowser, QDialog, QFont, QDesktopServices, QAction
+from PyQt4.QtCore import Qt, QSize, QTimer, QUrl
 from hivemind.dialogs import EditNodeDialog, EditLabelDialog, IconDialog, \
-AboutDialog, EditEdgeDialog, SettingsDialog, AffiliationDialog, NetworkSettingsDialog, \
-NetworkClientDialog, NetworkConnectionDialog
-from hivemind.gui_widgets import ScalingWidget, ActionImageButton, FindToolBar
+    AboutDialog, EditEdgeDialog, SettingsDialog, AffiliationDialog, NetworkSettingsDialog, \
+    NetworkClientDialog, NetworkConnectionDialog
+from hivemind.gui_widgets import ScalingWidget, ActionImageButton, FindToolBar, \
+    FilterToolBar, MaemoToolBar
 from hivemind.attribute import adaptCallable
 import hivemind.__version__ as __version__
 
@@ -339,8 +339,8 @@ class MaemoAffiliationDialog(MaemoDialogMixin, AffiliationDialog):
     Affiliation dialog for Maemo platform
     @author: Oleg Kandaurov
     '''
-    def __init__(self, model, parent = None):
-        AffiliationDialog.__init__(self, model, parent)
+    def __init__(self, model, rosterModel, parent = None):
+        AffiliationDialog.__init__(self, model, rosterModel, parent)
         MaemoDialogMixin.__init__(self)
 
 
@@ -555,39 +555,52 @@ class MaemoNetworkMainDialog(MaemoDialogMixin, QDialog):
         self.__networkSettingsButton.setEnabled(self.__bag.networkSettingsAction.isEnabled())
 
 
-class MaemoFindToolBar(FindToolBar):
+class MaemoFilterToolBar(MaemoToolBar, FilterToolBar):
     '''
-    Find tool bar for Desktop platform
+    Filter tool bar for Maemo platform
+    @author: Oleg Kandaurov
     '''
+    def __init__(self, controller, parent = None):
+        '''
+        @type controller: MainWindowController
+        @type parent: QWidget
+        '''
+        FilterToolBar.__init__(self, controller)
+        MaemoToolBar.__init__(self, parent)
+        FilterToolBar._createComponents(self)
+        self._initComponents()
+        FilterToolBar._createLayout(self)
 
-    __HEIGHT = 82
-    '''Height of the tool bar'''
+    def _initComponents(self):
+        '''Init components'''
+        self._closeAction.triggered.connect(self.close)
+        self._closeAction.setIcon(QIcon(resource.getImage('find_close')))
+        self._closeButton = ActionImageButton(self._closeAction)
+        self._findLineEdit.setStyleSheet("QLineEdit { border-width: 8px;"
+                 + " border-color: white; border-style: solid;"
+                 + " border-radius: 12px; background: white; }")
+        self.addWidget(self._findWidget)
 
-    __WIDTH = 630
-    '''Width of the tool bar'''
 
-    __RADIUS = 20
-    '''Corner radius'''
+class MaemoFindToolBar(MaemoToolBar, FindToolBar):
+    '''
+    Find tool bar for Maemo platform
+    @author: Eldar Mamedov, Oleg Kandaurov
+    '''
 
     def __init__(self, controller, parent = None):
         '''
         @type controller: MainWindowController
         @type parent: QWidget
         '''
-        FindToolBar.__init__(self, controller, parent)
+        FindToolBar.__init__(self, controller)
+        MaemoToolBar.__init__(self, parent)
         self._findWidget = QWidget()
-        self.resize(self.__WIDTH, self.__HEIGHT)
-        self.setAllowedAreas(Qt.NoToolBarArea)
-        self.setMovable(False)
-        self.setFloatable(False)
         self._createComponents()
         self._initComponents()
-        parent.installEventFilter(self)
 
     def _createComponents(self):
-        '''
-        Create components
-        '''
+        '''Create components'''
         #pylint: disable = W0201
         FindToolBar._createComponents(self)
         self.__prevAction = QAction(self)
@@ -604,9 +617,7 @@ class MaemoFindToolBar(FindToolBar):
         self.__closeButton = ActionImageButton(self.__closeAction)
 
     def _initComponents(self):
-        '''
-        Init components
-        '''
+        '''Init components'''
         buttonLayout = QHBoxLayout()
         buttonLayout.setSpacing(30)
         buttonLayout.addWidget(self.__prevButton)
@@ -623,42 +634,3 @@ class MaemoFindToolBar(FindToolBar):
                  + " border-color: white; border-style: solid;"
                  + " border-radius: 12px; background: white; }")
 
-    def paintEvent(self, paintEvent):
-        '''
-        Paint black background
-        @param paintEvent: paint event representation
-        @type paintEvent: QPaintEvent
-        '''
-        painter = QPainter(self)
-        background = QColor(self.backgroundRole())
-        background.setAlpha(128)
-        brush = QBrush(background)
-        pen = QPen(background)
-        painter.fillRect(0, self.__RADIUS, self.__WIDTH, self.__HEIGHT - self.__RADIUS, brush)
-        painter.fillRect(self.__RADIUS, 0, self.__WIDTH - self.__RADIUS, self.__RADIUS, brush)
-        painter.setPen(pen)
-        painter.setBrush(brush)
-        painter.drawPie(0, 0, (self.__RADIUS - 1) * 2, (self.__RADIUS - 1) * 2, 180 * 8,
-                180 * 8)
-
-    def eventFilter(self, obj, event):
-        '''
-        Catch Windows state changed and Resize events from parent object
-        @type obj: QEvent
-        @type event: QEvent
-        '''
-        parent = self.parentWidget()
-        if obj != self.parent():
-            return QToolBar.eventFilter(self, obj, event)
-        isFullScreen = parent.windowState() & Qt.WindowFullScreen
-        if event.type() == QEvent.Resize:
-            if isFullScreen:
-                self.resize(self.width() - parent.fullScreenExitButton.width(), self.__HEIGHT)
-                self.move(parent.width() - self.width() - parent.fullScreenExitButton.width(),
-                         parent.height() - parent.fullScreenExitButton.height())
-            else:
-                self.resize(self.__WIDTH, self.__HEIGHT)
-                self.move(parent.width() - self.width() + 10, parent.height() - self.height()
-                        + 10)
-        return QToolBar.eventFilter(self, obj, event)
-
