@@ -26,7 +26,7 @@ Abstract main window class
 
 import hivemind.settings as settings
 from PyQt4.QtGui import QPainter, QGraphicsView, QPalette, QMainWindow
-from PyQt4.QtCore import Qt, SIGNAL, QObject
+from PyQt4.QtCore import Qt, SIGNAL, pyqtSignal
 from hivemind.gui_widgets import SearchingTrMixin
 from hivemind.attribute import readable
 
@@ -50,10 +50,10 @@ class MainView(QGraphicsView):
         self.__draggedPath = 0
         self.__actionBag = actionBag
         self.__zoomLevel = 0
-        QObject.connect(self.__actionBag.zoomInAction, SIGNAL('triggered()'), self._zoomIn)
-        QObject.connect(self.__actionBag.zoomOutAction, SIGNAL('triggered()'), self._zoomOut)
+        self.__actionBag.zoomInAction.triggered.connect(self._zoomIn)
+        self.__actionBag.zoomOutAction.triggered.connect(self._zoomOut)
         self.__startPoint = None
-        QObject.connect(scene, SIGNAL('mapChanged'), self._resetView)
+        scene.mapChanged.connect(self._resetView)
 
     def scrollContentsBy(self, dx, dy):
         '''
@@ -167,6 +167,8 @@ class AbstractMainWindow(SearchingTrMixin, QMainWindow):
     @author: Alexander Kulikov
     '''
 
+    closeEvnt = pyqtSignal()
+
     def __init__(self, actionBag, mindMapScene):
         '''Create new window'''
         QMainWindow.__init__(self)
@@ -207,5 +209,5 @@ class AbstractMainWindow(SearchingTrMixin, QMainWindow):
 
     def closeEvent(self, event):
         '''Close event handler'''
-        self.emit(SIGNAL('closeEvent'))
+        self.closeEvnt.emit()
         event.ignore()
