@@ -29,9 +29,11 @@ from twisted import internet
 from twisted.words.protocols.jabber import jid
 from wokkel.client import HybridAuthenticator
 from wokkel.ping import PingHandler
+from hivemind.attribute import readable
 from idavoll.backend import BackendService
 from PyQt4.QtCore import QObject
-from hivemind.commands import StateModificationCommand, CurrentNodeCommand, ShutdownCommand
+from hivemind.commands import StateModificationCommand, CurrentNodeCommand, ShutdownCommand, \
+    AffiliationCommand
 from hivemind.network.models import AffiliationModel, RosterModel
 from hivemind.network import messages
 from hivemind.network import protocol_layer
@@ -68,28 +70,27 @@ class NetworkController(QObject):
         @type actionBag: ActionBag
         '''
         QObject.__init__(self)
+        self.__errorHandler = ErrorHandler(self)
+        self.__transport = Transport(self.__errorHandler)
         self.__mindMapController = mindMapController
         self.__actionBag = actionBag
         self.__affiliationModel = AffiliationModel()
-        self.__rosterModel = RosterModel()
-        self.__affiliationDialog = gui_factory.createAffiliationDialog(self.__affiliationModel)
+        self.__affiliationDialog = gui_factory.createAffiliationDialog(self.__affiliationModel,
+                self.__transport.rosterModel)
         self.__networkDialog = gui_factory.createNetworkConnectionDialog()
-        self.__clientDialog = gui_factory.createNetworkClientDialog(self.__rosterModel)
+        self.__clientDialog = gui_factory.createNetworkClientDialog(
+                self.__transport.rosterModel)
         self.__networkSettingsDialog = gui_factory.createNetworkSettingsDialog()
-        self.__streamMessages = messages.StreamErrorMessages()
-        self.__saslMessages = messages.SASLErrorMessages()
-        self.__stanzaMessages = messages.StanzaErrorMessages()
-        self.__pubsubMessages = messages.PubSubErrorMessages()
+
         self.__isServiceRole = False
-        self.__xmppClient = None
         self.__protocol = None
-        self.__rosterProtocol = None
         self.__pingHandler = None
         self.__storage = None
         self.__secondConnectionStep = None
-        self.__pingManager = None
+        self.__clientRole = None
+        self.__serverPublishModel = 'publishers'
         self._connectActions()
-        self._updateActions(connectionState.disconnected)
+        self.updateActions(connectionState.disconnected)
         settings.connect(self._updateConfiguration)
 
     def publishCommand(self, command):
@@ -97,12 +98,24 @@ class NetworkController(QObject):
         Publish command in common space
         @param command: command to publish in shared space
         '''
+        def isClientPropagationAllowed():
+            '''Whether client can publish or not'''
+            if (self.__clientRole in ['outcast', 'member'] and
+                    self.__serverPublishModel == 'publishers'):
+                return False
+            return True
+
         if self.__protocol:
-            if _isPropagationAllowed(command):
-                self.__protocol.proposeChangeset(Changeset(parser.serializeCommand(command),
-                        'command'))
+            if self.__isServiceRole == True:
+                if _isPropagationAllowed(command):
+                    self.__protocol.proposeChangeset(
+                        Changeset(parser.serializeCommand(command), 'command'))
+                else:
+                    self.__mindMapController.executeCommand(command, False)
             else:
-                self.__mindMapController.executeCommand(command, False)
+                if isClientPropagationAllowed():
+                    self.__protocol.proposeChangeset(
+                        Changeset(parser.serializeCommand(command), 'command'))
         else:
             self.__mindMapController.executeCommand(command)
 
@@ -115,7 +128,7 @@ class NetworkController(QObject):
         if not self.__mindMapController.prohibitChangeMapAction():
             return
         reactor.callLater(SHUTDOWN_TIMEOUT, stopReactor, self)
-        defer = self._closeNetwork()
+        defer = self.closeNetwork()
         defer.addCallback(stopReactor)
 
     def _connectActions(self):
@@ -135,8 +148,8 @@ class NetworkController(QObject):
             else:
                 self.__networkDialog.exec_()
 
-        self.__actionBag.disconnectXMPPAction.triggered.connect(self._closeNetwork)
-        self.__actionBag.stopProtocolAction.triggered.connect(self._closeProtocolLevel)
+        self.__actionBag.disconnectXMPPAction.triggered.connect(self.closeNetwork)
+        self.__actionBag.stopProtocolAction.triggered.connect(self.closeProtocol)
         self.__actionBag.startServiceAction.triggered.connect(prepareService)
         self.__actionBag.startServiceAction.triggered.connect(showNetworkDialog)
         self.__actionBag.startClientAction.triggered.connect(prepareClient)
@@ -149,7 +162,7 @@ class NetworkController(QObject):
         self.__clientDialog.accepted.connect(self._startClient)
         self.__affiliationDialog.accepted.connect(self._updateAffiliationsFromModel)
 
-    def _updateActions(self, state):
+    def updateActions(self, state):
         '''
         Update actions according to state
         @type state: str
@@ -182,10 +195,14 @@ class NetworkController(QObject):
             defer = self.__storage.getNode(XMPP_NODE_NAME)
             defer.addCallback(lambda node:
                     node.setAffiliation(jid.internJID(entity), affiliation))
+            command = AffiliationCommand(entity, affiliation)
+            self.__protocol.proposeChangeset(
+                    Changeset(parser.serializeCommand(command), 'command'))
 
     def _startNetwork(self):
         '''Start network connection'''
-        self._startTransport(self.__networkDialog.attributes['userJid'] + '/hivemind',
+        self.__transport.setController(self)
+        self.__transport.start(self.__networkDialog.attributes['userJid'] + '/hivemind',
                 self.__networkDialog.attributes['password'],
                 self.__networkDialog.attributes['host'],
                 self.__networkDialog.attributes['port'])
@@ -203,28 +220,11 @@ class NetworkController(QObject):
 
     def _performSecondConnectionStep(self):
         ''' Perform second connection step'''
-        if self.__secondConnectionStep and self.__xmppClient is not None:
+        if self.__secondConnectionStep and self.__transport.xmppClient is not None:
             {'client': self.__clientDialog.exec_,
                         'service': self._startService}[self.__secondConnectionStep]()
         self.__secondConnectionStep = None
 
-    def _startTransport(self, userJid, password, host, port):
-        '''
-        Start XMPP connection
-        @param userJid: JID for XMPP connection
-        @type userJid: str
-        @param password: password for XMPP connection
-        @type password: str
-        @type host: str
-        @type port: int
-        '''
-        authenticator = HybridAuthenticator(jid.internJID(userJid), password)
-        factory = HivemindXmlStreamFactory(authenticator)
-        factory.addObserver(self)
-        self.__xmppClient = HivemindXMPPClient(authenticator, factory, host, port)
-        self.__xmppClient.addObserver(self)
-        self.__xmppClient.startService()
-
     def _configureService(self):
         '''Configure HiveMind service'''
         self.__isServiceRole = True
@@ -233,7 +233,7 @@ class NetworkController(QObject):
         resource = HivemindResourceFromBackend(backend)
         self.__protocol = HivemindPubSubService(resource)
         resource.pubsubService = self.__protocol
-        resource.serviceJID = self.__xmppClient.jid
+        resource.serviceJID = self.__transport.xmppClient.jid
         mindMap = parser.mindMapToString(self.__mindMapController.initialMindMap, False)
         undoStack = parser.serializeCommandStack(self.__mindMapController.undoStack)
         backend.createNode(XMPP_NODE_NAME, resource.serviceJID)
@@ -254,74 +254,40 @@ class NetworkController(QObject):
 
     def _startProtocol(self):
         '''Start Hivemind protocol'''
-        if not self.__xmppClient:
-            return
         self.__protocol.addObserver(self)
-        self.__protocol.setHandlerParent(self.__xmppClient)
-
-    def _startTransportHelpers(self):
-        '''Start transport helpers'''
-        if not self.__xmppClient:
-            return
-        self.__rosterProtocol = HivemindRosterClientProtocol()
-        self.__rosterProtocol.addObserver(self)
-        self.__rosterProtocol.setHandlerParent(self.__xmppClient)
-        self.__pingManager = PingManager(self, self.__xmppClient)
-        self.__pingManager.addPinger(self.__xmppClient.server,
-                settings.get('transportPingInterval'),
-                settings.get('transportPingMaxCount'), isServer = True)
-        self.__pingManager.startPing(self.__xmppClient.server)
+        self.__protocol.setHandlerParent(self.__transport.xmppClient)
 
     def _startProtocolHelpers(self):
         '''Start protocol helpers'''
-        if not self.__xmppClient:
-            return
         self.__pingHandler = PingHandler()
-        self.__pingHandler.setHandlerParent(self.__xmppClient)
+        self.__pingHandler.setHandlerParent(self.__transport.xmppClient)
 
-    def _closeNetwork(self):
+    def closeNetwork(self):
         '''Closes network connection'''
-        defer = self._closeProtocolLevel()
-        defer.addCallback(lambda success: self._closeTransportLevel())
+        defer = self.closeProtocol()
+        defer.addCallback(lambda success: self.__transport.close())
         return defer
 
-    def _closeTransportLevel(self):
-        ''' Close transport level'''
-        if self.__rosterProtocol:
-            if self.__xmppClient:
-                self.__rosterProtocol.disownHandlerParent(self.__xmppClient)
-            self.__rosterProtocol.deleteObserver(self)
-            self.__rosterProtocol = None
-        if self.__pingManager:
-            if self.__xmppClient:
-                self.__pingManager.stopPing(self.__xmppClient.server)
-            self.__pingManager.removeAll()
-            self.__pingManager = None
-        if self.__xmppClient:
-            self.__xmppClient.stopService()
-            self.__xmppClient = None
-        self._updateActions(connectionState.disconnected)
-
-    def _closeProtocolLevel(self):
+    def closeProtocol(self):
         '''Close protocol level'''
         def destroyProtocol(result):
             '''Destroy protocol'''
-            if self.__xmppClient:
-                self.__protocol.disownHandlerParent(self.__xmppClient)
+            if self.__transport.xmppClient:
+                self.__protocol.disownHandlerParent(self.__transport.xmppClient)
             self.__protocol.deleteObserver(self)
             self.__protocol = None
             return result
-
-        if self.__xmppClient:
+        if self.__transport is None: return
+        if self.__transport.xmppClient:
             if self.__pingHandler:
-                self.__pingHandler.disownHandlerParent(self.__xmppClient)
+                self.__pingHandler.disownHandlerParent(self.__transport.xmppClient)
                 self.__pingHandler = None
-            if self.__pingManager:
-                self.__pingManager.removeAll([self.__xmppClient.server])
+            if self.__transport.pingManager:
+                self.__transport.pingManager.removeAll([self.__transport.xmppClient.server])
         defer = internet.defer.succeed(None)
-        if self.__protocol is None:
+        if self.__protocol is None or self.__transport.xmppClient is None:
             return defer
-        if self.__isServiceRole and self.__protocol.jid:
+        if self.__isServiceRole:
             defer = self.__protocol.proposeChangeset(
                     Changeset(parser.serializeCommand(ShutdownCommand()), 'command'))
         defer.addBoth(self.__protocol.close)
@@ -333,50 +299,94 @@ class NetworkController(QObject):
         for event in kvargs.keys():
             getattr(self, '_process%s%sEvent' % (event[0].upper(), event[1:]))(kvargs[event])
 
+    def _processErrorEvent(self, failure):
+        '''Process error event'''
+        self.__errorHandler.mapErrorEvent(failure)
+
     def _processAffiliationEvent(self, event):
         '''Process affiliation event from node'''
         entity, affiliation = event
         self.__affiliationModel.updateAffiliation(entity, affiliation)
 
-    def _processTransportPingEvent(self, event):
-        '''Process ping event from transport'''
-        entity = event['entity']
-        state = event['state']
-        if state == Pinger.entityState.timeout:
-            self._processConnectionEvent(connectionState.disconnected)
-            if self.__pingManager:
-                self.__pingManager.stopPing(entity)
-            #Do tricky reconnect
-            if self.__xmppClient:
-                self.__xmppClient.factory.resetDelay()
-                self.__xmppClient.factory.retry(self.__xmppClient.connection)
-
     def _processProtocolPingEvent(self, event):
         '''Process ping event from protocol'''
         entity = event['entity']
         state = event['state']
         if state == Pinger.entityState.timeout:
-            if self.__protocol and self.__xmppClient:
+            if self.__protocol and self.__transport.xmppClient:
                 # seems broken, need inspection
-                self.__protocol.unsubscribe(entity, self.__xmppClient.jid,
+                self.__protocol.unsubscribe(entity, self.__transport.xmppClient.jid,
                             XMPP_NODE_NAME, entity)
-            if self.__pingManager:
-                self.__pingManager.stopPing(entity)
+            if self.__transport.pingManager:
+                self.__transport.pingManager.stopPing(entity)
 
-    def _processRosterEvent(self, event):
-        '''Process roster event'''
-        mapEvt = {HivemindRosterClientProtocol.eventType.roster: self.__rosterModel.setRoster,
-         HivemindRosterClientProtocol.eventType.remove: self.__rosterModel.removeContact,
-         HivemindRosterClientProtocol.eventType.set: self.__rosterModel.setContact}
-        processEvent = mapEvt[event['type']]
-        processEvent(event['data'])
+    def _processSubscriptionEvent(self, event):
+        '''Process subscription event'''
+        entity = event['entity']
+        state = event['state']
+        self.__affiliationModel.updateSubscriptionState(entity, state)
+        if self.__transport.pingManager is None: return
+        if state == subscriptionState.subscribed:
+            self.__transport.pingManager.addPinger(entity,
+                    settings.get('protocolPingInterval'),
+                    settings.get('protocolPingMaxCount'))
+            self.__transport.pingManager.startPing(entity)
+        else:
+            self.__transport.pingManager.removePinger(entity)
+
+    def _processConnectionEvent(self, state):
+        '''Process connection event'''
+        mapConnectionState = {connectionState.initialized: self.__transport.startHelpers,
+                connectionState.collaboration:  self._startProtocolHelpers}
+        processEvent = mapConnectionState[state]
+        if processEvent is not None:
+            processEvent()
+        self.updateActions(state)
+
+    def _processChangesetEvent(self, changeset):
+        '''Process changeset event'''
+        if (self.__isServiceRole and changeset.type != 'command'):
+            return
+        data = self.__mindMapController.deserializeData(changeset.data, changeset.type)
+        if not self.__isServiceRole:
+            if isinstance(data, ShutdownCommand):
+                self.closeProtocol()
+                return
+            elif isinstance(data, AffiliationCommand):
+                if data.entity == self.__transport.xmppClient.jid.userhost():
+                    self.__clientRole = data.affiliation
+                return
+        mapChangesetType = {'mindmap': 'loadMindMap',
+                'command': 'executeCommand',
+                'undostack': 'loadUndoStack'}
+        processChangeset = mapChangesetType[changeset.type]
+        getattr(self.__mindMapController, processChangeset)(data)
+
+
+class ErrorHandler(QObject):
+    '''
+    Handles network errors
+    @author: Oleg Kandaurov
+    '''
+
+    def __init__(self, networkCtrl):
+        """
+        Constructor
+        @type networkCtrl: NetworkController
+        """
+        QObject.__init__(self)
+        self.__networkCtrl = networkCtrl
+        self.__streamMessages = messages.StreamErrorMessages()
+        self.__saslMessages = messages.SASLErrorMessages()
+        self.__stanzaMessages = messages.StanzaErrorMessages()
+        self.__pubsubMessages = messages.PubSubErrorMessages()
 
     def _handleSASLError(self, failure):
         '''
         Handle sasl error
         @type failure: Failure
         '''
-        errorDialog = gui_factory.createNetworkErrorDialog(self._closeNetwork)
+        errorDialog = gui_factory.createNetworkErrorDialog(self.__networkCtrl.closeNetwork)
         errorDialog.showError(self.tr('Authentication fails.'),
                 self.tr('Seems that you provide invalid username or password. ' +
                 'See details for more info.'),
@@ -387,7 +397,7 @@ class NetworkController(QObject):
         Handle stream error
         @type failure: Failure
         '''
-        errorDialog = gui_factory.createNetworkErrorDialog(self._closeNetwork)
+        errorDialog = gui_factory.createNetworkErrorDialog(self.__networkCtrl.closeNetwork)
         errorDialog.showError(self.tr('XMPP connection fails.'),
                 self.tr('The error is nonrecoverable. ' +
                 'You are automatically disconnected. ' +
@@ -399,7 +409,7 @@ class NetworkController(QObject):
         Handle stanza error
         @type failure: Failure
         '''
-        errorDialog = gui_factory.createNetworkErrorDialog(self._closeProtocolLevel)
+        errorDialog = gui_factory.createNetworkErrorDialog(self.__networkCtrl.closeProtocol)
         errorDialog.showError(self.tr('Collaboration error has been occured.'),
                 self.tr('Fortunately, the error is not fatal. ' +
                 'Often this means that you are not allowed to do some action. ' +
@@ -411,7 +421,7 @@ class NetworkController(QObject):
         Handle pubsub error
         @type failure: Failure
         '''
-        errorDialog = gui_factory.createNetworkErrorDialog(self._closeProtocolLevel)
+        errorDialog = gui_factory.createNetworkErrorDialog(self.__networkCtrl.closeProtocol)
         condition, feature = protocol_layer.pubsubConditionFromStanzaError(failure.value)
         message = self.__stanzaMessages.condition[failure.value.condition] + \
                 self.tr('\nCondition: %s') % (
@@ -425,9 +435,9 @@ class NetworkController(QObject):
                 'See details for more info.'),
                 message, False)
 
-    def _processErrorEvent(self, failure):
+    def mapErrorEvent(self, failure):
         ''''
-        Process network errors
+        Map different network errors
         @type failure: Failure
         '''
         error = failure.trap(jabber.sasl.SASLError, jabber.error.StanzaError,
@@ -442,46 +452,124 @@ class NetworkController(QObject):
             else:
                 self._handleStanzaError(failure)
 
-    def _processSubscriptionEvent(self, event):
-        '''Process subscription event'''
-        entity = event['entity']
+
+class Transport(object):
+    '''
+    Manages XMPP connection
+
+    @author: Oleg Kandaurov
+    '''
+
+    readable('xmppClient', 'pingManager', 'rosterModel')
+    def __init__(self, errorHandler):
+        '''Constructor'''
+        self.__errorHandler = errorHandler
+        self.__rosterModel = RosterModel()
+        self.__controller = None
+        self.__rosterProtocol = None
+        self.__pinger = None
+        self.__pingManager = None
+        self.__xmppClient = None
+
+    def setController(self, controller):
+        '''Setter'''
+        self.__controller = controller
+
+    def start(self, userJid, password, host, port):
+        '''
+        Start XMPP connection
+        @param userJid: JID for XMPP connection
+        @type userJid: str
+        @param password: password for XMPP connection
+        @type password: str
+        @type host: str
+        @type port: int
+        '''
+        authenticator = HybridAuthenticator(jid.internJID(userJid), password)
+        factory = HivemindXmlStreamFactory(authenticator)
+        factory.addObserver(self)
+        self.__xmppClient = HivemindXMPPClient(authenticator, factory, host, port)
+        self.__xmppClient.addObserver(self)
+        self.__xmppClient.startService()
+
+    def startHelpers(self):
+        '''Start transport helpers'''
+        self.__rosterProtocol = HivemindRosterClientProtocol()
+        self.__rosterProtocol.addObserver(self)
+        self.__rosterProtocol.setHandlerParent(self.__xmppClient)
+        self.__pingManager = PingManager(self, self.__xmppClient)
+        self.__pinger = Pinger(self.__xmppClient.server,
+                settings.get('transportPingInterval'),
+                settings.get('transportPingMaxCount'), isServer = True)
+        self.__pinger.setHandlerParent(self.__xmppClient)
+        self.__pinger.addObserver(self)
+        self.__pinger.startPing()
+
+    def close(self):
+        ''' Close transport level'''
+        if self.__rosterProtocol:
+            if self.__xmppClient:
+                self.__rosterProtocol.disownHandlerParent(self.__xmppClient)
+            self.__rosterProtocol.deleteObserver(self)
+            self.__rosterProtocol = None
+        if self.__pingManager:
+            if self.__xmppClient:
+                self.__pingManager.removeAll()
+                self.__pingManager = None
+        if self.__pinger:
+            self.__pinger.stopPing()
+            self.__pinger.disownHandlerParent(self.__xmppClient)
+            self.__pinger = None
+        if self.__xmppClient:
+            self.__xmppClient.stopService()
+            self.__xmppClient = None
+        if self.__controller is not None:
+            self.__controller.updateActions(connectionState.disconnected)
+
+    def processNotification(self, *values, **kvargs):
+        '''Process notification'''
+        for event in kvargs.keys():
+            getattr(self, '_process%s%sEvent' % (event[0].upper(), event[1:]))(kvargs[event])
+
+    def _processErrorEvent(self, failure):
+        '''Process error event'''
+        self.__errorHandler.mapErrorEvent(failure)
+
+    def _processTransportPingEvent(self, event):
+        '''Process ping event from transport'''
         state = event['state']
-        self.__affiliationModel.updateSubscriptionState(entity, state)
-        if self.__pingManager is None: return
-        if state == subscriptionState.subscribed:
-            self.__pingManager.addPinger(entity,
-                    settings.get('protocolPingInterval'),
-                    settings.get('protocolPingMaxCount'))
-            self.__pingManager.startPing(entity)
-        else:
-            self.__pingManager.removePinger(entity)
+        if state == Pinger.entityState.timeout:
+            self._processConnectionEvent(connectionState.disconnected)
+            if self.__pinger:
+                self.__pinger.stopPing()
+            if self.__xmppClient:
+                self._reconnect()
+
+    def _processRosterEvent(self, event):
+        '''Process roster event'''
+        mapEvt = {HivemindRosterClientProtocol.eventType.roster: self.__rosterModel.setRoster,
+         HivemindRosterClientProtocol.eventType.remove: self.__rosterModel.removeContact,
+         HivemindRosterClientProtocol.eventType.set: self.__rosterModel.setContact}
+        processEvent = mapEvt[event['type']]
+        processEvent(event['data'])
 
     def _processConnectionEvent(self, state):
         '''Process connection event'''
-        mapConnectionState = {connectionState.collaboration:  self._startProtocolHelpers,
-                              connectionState.initialized: self._startTransportHelpers,
-                              connectionState.disconnected:  self._closeNetwork,
+        mapConnectionState = {connectionState.initialized: self.startHelpers,
+                              connectionState.disconnected: self.__controller.closeNetwork,
                               connectionState.connecting: None,
                               connectionState.connected: None}
         processEvent = mapConnectionState[state]
         if processEvent is not None:
             processEvent()
-        self._updateActions(state)
+        self.__controller.updateActions(state)
 
-    def _processChangesetEvent(self, changeset):
-        '''Process changeset event'''
-        if (self.__isServiceRole and changeset.type != 'command'):
-            return
-        data = self.__mindMapController.deserializeData(changeset.data, changeset.type)
-        if isinstance(data, ShutdownCommand):
-            if not self.__isServiceRole:
-                self._closeProtocolLevel()
-            return
-        mapChangesetType = {'mindmap': 'loadMindMap',
-                'command': 'executeCommand',
-                'undostack': 'loadUndoStack'}
-        processChangeset = mapChangesetType[changeset.type]
-        getattr(self.__mindMapController, processChangeset)(data)
+    def _reconnect(self):
+        '''
+        Perform reconnect. Implementation is somewhat tricky.
+        '''
+        self.__xmppClient.factory.resetDelay()
+        self.__xmppClient.factory.retry(self.__xmppClient.connection)
 
 
 def startReactor():
@@ -492,7 +580,7 @@ def startReactor():
     # stop twisted event loop when last window is closed
     reactor.qApp.lastWindowClosed.connect(reactor.stop)
 
-    
+
 def _isPropagationAllowed(command):
     '''
     Check if command must be propagated through network or not
